@@ -42,14 +42,13 @@ if [ ! -v BASEDB ]
 then
     exit_fail "-d option required"
 fi
-
 echo -n "Testing for curl ..."
 curl --help >/dev/null </dev/null || exit_skip " MISSING"
 echo " FOUND"
 
 # reset database
 echo -n "Reset 'auditor-basedb' database at $PGHOST ..."
-dropdb "auditor-basedb" >/dev/null 2>/dev/null || true
+dropdb --if-exists "auditor-basedb" > /dev/null 2> /dev/null || true
 createdb "auditor-basedb" || exit_skip "Could not create database '$BASEDB' at $PGHOST"
 echo " DONE"
 
@@ -63,7 +62,7 @@ EXCHANGE_URL=$(taler-config -c "$CONF" -s EXCHANGE -o BASE_URL)
 MERCHANT_PORT=$(taler-config -c "$CONF" -s MERCHANT -o PORT)
 MERCHANT_URL="http://localhost:${MERCHANT_PORT}/"
 BANK_PORT=$(taler-config -c "$CONF" -s BANK -o HTTP_PORT)
-BANK_URL="http://localhost:${BANK_PORT}"
+BANK_URL="http://localhost:${BANK_PORT}/"
 
 echo -n "Checking setup worked ..."
 wget \
