@@ -99,8 +99,8 @@ echo "Testing for jq"
 jq -h > /dev/null || exit_skip "jq required"
 echo "Testing for faketime"
 faketime -h > /dev/null || exit_skip "faketime required"
-echo "Testing for libeufin"
-libeufin-cli --help >/dev/null </dev/null 2> /dev/null || exit_skip "libeufin required"
+echo "Testing for libeufin-bank"
+libeufin-bank --help >/dev/null </dev/null 2> /dev/null || exit_skip "libeufin-bank required"
 echo "Testing for pdflatex"
 which pdflatex > /dev/null </dev/null || exit_skip "pdflatex required"
 echo "Testing for taler-wallet-cli"
