@@ -42,11 +42,6 @@
 struct PlanchetContext
 {
 
-  /**
-   * Hash of the (blinded) message to be signed by the Exchange.
-   */
-  struct TALER_BlindedCoinHashP h_coin_envelope;
-
   /**
    * Value of the coin being exchanged (matching the denomination key)
    * plus the transaction fee.  We include this in what is being
@@ -258,22 +253,23 @@ check_request_idempotent (const struct BatchWithdrawContext *wc,
   {
     struct PlanchetContext *pc = &wc->planchets[i];
     enum GNUNET_DB_QueryStatus qs;
+    struct TALER_EXCHANGEDB_CollectableBlindcoin collectable;
 
     qs = TEH_plugin->get_withdraw_info (TEH_plugin->cls,
-                                        &pc->h_coin_envelope,
-                                        &pc->collectable);
+                                        &pc->collectable.h_coin_envelope,
+                                        &collectable);
     if (0 > qs)
     {
       GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
-      if (GNUNET_DB_STATUS_HARD_ERROR == qs)
-        *mret = TALER_MHD_reply_with_error (rc->connection,
-                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                            TALER_EC_GENERIC_DB_FETCH_FAILED,
-                                            "get_withdraw_info");
+      *mret = TALER_MHD_reply_with_error (rc->connection,
+                                          MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                          TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                          "get_withdraw_info");
       return true; /* well, kind-of */
     }
     if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
       return false;
+    pc->collectable = collectable;
   }
   /* generate idempotent reply */
   TEH_METRICS_num_requests[TEH_MT_REQUEST_IDEMPOTENT_BATCH_WITHDRAW]++;
@@ -510,7 +506,6 @@ batch_withdraw_transaction (void *cls,
       &TEH_age_restriction_config.mask,
       allowed_maximum_age);
 
-    TEH_plugin->rollback (TEH_plugin->cls);
     *mhd_ret = TEH_RESPONSE_reply_reserve_age_restriction_required (
       connection,
       lowest_age);
@@ -519,7 +514,12 @@ batch_withdraw_transaction (void *cls,
 
   if (! balance_ok)
   {
-    TEH_plugin->rollback (TEH_plugin->cls);
+    if (check_request_idempotent (wc,
+                                  mhd_ret))
+    {
+      return GNUNET_DB_STATUS_HARD_ERROR;
+    }
+
     *mhd_ret = TEH_RESPONSE_reply_reserve_insufficient_balance (
       connection,
       TALER_EC_EXCHANGE_WITHDRAW_INSUFFICIENT_FUNDS,
@@ -689,7 +689,6 @@ parse_planchets (const struct TEH_RequestContext *rc,
                  const json_t *planchets)
 {
   struct TEH_KeyStateHandle *ksh;
-  MHD_RESULT mret;
 
   for (unsigned int i = 0; i<wc->planchets_length; i++)
   {
@@ -735,6 +734,8 @@ parse_planchets (const struct TEH_RequestContext *rc,
   ksh = TEH_keys_get_state ();
   if (NULL == ksh)
   {
+    MHD_RESULT mret;
+
     if (! check_request_idempotent (wc,
                                     &mret))
     {
@@ -758,6 +759,8 @@ parse_planchets (const struct TEH_RequestContext *rc,
 
     if (NULL == dk)
     {
+      MHD_RESULT mret;
+
       if (! check_request_idempotent (wc,
                                       &mret))
       {
@@ -769,6 +772,8 @@ parse_planchets (const struct TEH_RequestContext *rc,
     }
     if (GNUNET_TIME_absolute_is_past (dk->meta.expire_withdraw.abs_time))
     {
+      MHD_RESULT mret;
+
       /* This denomination is past the expiration time for withdraws */
       if (! check_request_idempotent (wc,
                                       &mret))
@@ -793,6 +798,8 @@ parse_planchets (const struct TEH_RequestContext *rc,
     }
     if (dk->recoup_possible)
     {
+      MHD_RESULT mret;
+
       /* This denomination has been revoked */
       if (! check_request_idempotent (wc,
                                       &mret))
