@@ -374,6 +374,7 @@ TEH_handler_purses_deposit (
     struct GNUNET_TIME_Timestamp create_timestamp;
     struct GNUNET_TIME_Timestamp merge_timestamp;
     bool was_deleted;
+    bool was_refunded;
 
     qs = TEH_plugin->select_purse (
       TEH_plugin->cls,
@@ -384,7 +385,8 @@ TEH_handler_purses_deposit (
       &pcc.deposit_total,
       &pcc.h_contract_terms,
       &merge_timestamp,
-      &was_deleted);
+      &was_deleted,
+      &was_refunded);
     switch (qs)
     {
     case GNUNET_DB_STATUS_HARD_ERROR:
@@ -407,7 +409,7 @@ TEH_handler_purses_deposit (
     case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
       break; /* handled below */
     }
-    if (GNUNET_TIME_absolute_is_past (pcc.purse_expiration.abs_time) ||
+    if (was_refunded ||
         was_deleted)
     {
       return TALER_MHD_reply_with_error (
