@@ -362,7 +362,6 @@ exchange_serve_process_config (void)
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "No accounts enabled for credit!\n");
     GNUNET_SCHEDULER_shutdown ();
-    global_ret = EXIT_INVALIDARGUMENT;
     return GNUNET_SYSERR;
   }
   return GNUNET_OK;
@@ -490,7 +489,7 @@ transaction_completed (void)
                                           latency);
     if (! (test_mode ||
            GNUNET_TIME_relative_is_zero (left)) )
-      GNUNET_log (GNUNET_ERROR_TYPE_ERROR, // WARNING,
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                   "Server did not respect long-polling, enforcing client-side by sleeping for %s\n",
                   GNUNET_TIME_relative2s (left,
                                           true));
@@ -630,7 +629,7 @@ process_reply (const struct TALER_BANK_CreditDetails *details,
         break;
       case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
         GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                    "Imported transaction %llu.",
+                    "Imported transaction %llu.\n",
                     (unsigned long long) cd->serial_id);
         /* normal case */
         progress = true;
@@ -951,6 +950,7 @@ run (void *cls,
   {
     GNUNET_break (0);
     GNUNET_SCHEDULER_shutdown ();
+    global_ret = EXIT_NO_RESTART;
     return;
   }
   rc = GNUNET_CURL_gnunet_rc_create (ctx);
