@@ -80,14 +80,9 @@ get_attributes_cb (void *cls,
     size_t enc_attributes_size;
     void *enc_attributes;
     char *provider;
-    char *birthdate = NULL;
     struct GNUNET_PQ_ResultSpec rs[] = {
       GNUNET_PQ_result_spec_string ("provider",
                                     &provider),
-      GNUNET_PQ_result_spec_allow_null (
-        GNUNET_PQ_result_spec_string ("birthdate",
-                                      &birthdate),
-        NULL),
       GNUNET_PQ_result_spec_timestamp ("collection_time",
                                        &collection_time),
       GNUNET_PQ_result_spec_timestamp ("expiration_time",
@@ -110,7 +105,6 @@ get_attributes_cb (void *cls,
     ctx->cb (ctx->cb_cls,
              ctx->h_payto,
              provider,
-             birthdate,
              collection_time,
              expiration_time,
              enc_attributes_size,
@@ -145,7 +139,6 @@ TEH_PG_select_kyc_attributes (
            "select_kyc_attributes",
            "SELECT "
            " provider"
-           ",birthdate"
            ",collection_time"
            ",expiration_time"
            ",encrypted_attributes"
