@@ -153,6 +153,33 @@ TALER_JSON_pack_blinded_planchet (
   const struct TALER_BlindedPlanchet *blinded_planchet);
 
 
+/**
+  * Generate packer instruction for a JSON field of type
+  * blinded token issue signature.
+  *
+  * @param name name of the field to add to the object
+  * @param sig token issue signature
+  * @return json pack specification
+  */
+struct GNUNET_JSON_PackSpec
+TALER_JSON_pack_token_issue_sig (
+  const char *name,
+  const struct TALER_TokenIssueSignatureP *sig);
+
+
+/**
+  * Generate packer instruction for a JSON field of type token envelope.
+  *
+  * @param name name of the field to add to the object
+  * @param envelope blinded token envelope
+  * @return json pack specification
+ */
+struct GNUNET_JSON_PackSpec
+TALER_JSON_pack_token_envelope (
+  const char *name,
+  const struct TALER_TokenEnvelopeP *envelope);
+
+
 /**
  * Generate packer instruction for a JSON field of type
  * exchange withdraw values (/csr).
@@ -587,6 +614,45 @@ TALER_JSON_spec_i18n_str (const char *name,
                           const char **strptr);
 
 
+/**
+ * Generate line in parser specification for a
+ * token issue signature.
+ *
+ * @param field name of the field
+ * @param[out] sig the signature to initialize
+ * @return corresponding field spec
+ */
+struct GNUNET_JSON_Specification
+TALER_JSON_spec_token_issue_sig (const char *field,
+                                 struct TALER_TokenIssueSignatureP *sig);
+
+
+/**
+ * Generate line in parser specification for a
+ * blinded token issue signature.
+ *
+ * @param field name of the field
+ * @param[out] sig the blinded signature to initialize
+ * @return corresponding field spec
+ */
+struct GNUNET_JSON_Specification
+TALER_JSON_spec_blinded_token_issue_sig (
+  const char *field,
+  struct TALER_TokenIssueBlindSignatureP *sig);
+
+/**
+ * Generate line in parser specification for a
+ * token envelope.
+ *
+ * @param field name of the field
+ * @param[out] env the envelope to initialize
+ * @return corresponding field spec
+ */
+struct GNUNET_JSON_Specification
+TALER_JSON_spec_token_envelope (const char *field,
+                                struct TALER_TokenEnvelopeP *env);
+
+
 /**
  * Hash a JSON for binary signing.
  *
