@@ -57,16 +57,16 @@
 
 /* Define logging functions */
 #define TALER_LOG_DEBUG(...)                                  \
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, __VA_ARGS__)
+        GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, __VA_ARGS__)
 
 #define TALER_LOG_INFO(...)                                  \
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO, __VA_ARGS__)
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO, __VA_ARGS__)
 
 #define TALER_LOG_WARNING(...)                                \
-  GNUNET_log (GNUNET_ERROR_TYPE_WARNING, __VA_ARGS__)
+        GNUNET_log (GNUNET_ERROR_TYPE_WARNING, __VA_ARGS__)
 
 #define TALER_LOG_ERROR(...)                                  \
-  GNUNET_log (GNUNET_ERROR_TYPE_ERROR, __VA_ARGS__)
+        GNUNET_log (GNUNET_ERROR_TYPE_ERROR, __VA_ARGS__)
 
 
 /**
@@ -77,11 +77,11 @@
  * @param reason string to print as warning
  */
 #define TALER_assert_as(EXP, reason)                           \
-  do {                                                          \
-    if (EXP) break;                                             \
-    TALER_LOG_ERROR ("%s at %s:%d\n", reason, __FILE__, __LINE__);       \
-    abort ();                                                    \
-  } while (0)
+        do {                                                          \
+          if (EXP) break;                                             \
+          TALER_LOG_ERROR ("%s at %s:%d\n", reason, __FILE__, __LINE__);       \
+          abort ();                                                    \
+        } while (0)
 
 
 /**
@@ -112,10 +112,11 @@
 
 
 #define TALER_gcry_ok(cmd) \
-  do {int rc; rc = cmd; if (! rc) break; \
-      TALER_LOG_ERROR ("A Gcrypt call failed at %s:%d with error: %s\n", \
-                       __FILE__, \
-                       __LINE__, gcry_strerror (rc)); abort (); } while (0)
+        do {int rc; rc = cmd; if (! rc) break; \
+            TALER_LOG_ERROR ("A Gcrypt call failed at %s:%d with error: %s\n", \
+                             __FILE__, \
+                             __LINE__, gcry_strerror (rc)); abort (); } while (0 \
+                                                                               )
 
 
 /**
@@ -259,6 +260,7 @@ struct TALER_CurrencySpecification
  * our configuration.
  *
  * @param cfg configuration to parse
+ * @param main_currency main currency of the component
  * @param[out] num_currencies set to number of enabled currencies, length of @e cspecs
  * @param[out] cspecs set to currency specification array
  * @return #GNUNET_OK on success, #GNUNET_NO if zero
@@ -267,6 +269,7 @@ struct TALER_CurrencySpecification
  */
 enum GNUNET_GenericReturnValue
 TALER_CONFIG_parse_currencies (const struct GNUNET_CONFIGURATION_Handle *cfg,
+                               const char *main_currency,
                                unsigned int *num_currencies,
                                struct TALER_CurrencySpecification **cspecs);
 
@@ -771,7 +774,7 @@ TALER_get_lowest_age (
  * @return lowest age for the largest age group
  */
 #define TALER_adult_age(mask) \
-  sizeof((mask)->bits) * 8 - __builtin_clz ((mask)->bits) - 1
+        sizeof((mask)->bits) * 8 - __builtin_clz ((mask)->bits) - 1
 
 /**
  * Handle to an external process that will assist
