@@ -531,6 +531,35 @@ TALER_JSON_spec_age_commitment (const char *name,
   return ret;
 }
 
+struct GNUNET_JSON_Specification
+TALER_JSON_spec_token_issue_sig (const char *field,
+                                 struct TALER_TokenIssueSignatureP *sig)
+{
+  sig->signature = NULL;
+  return GNUNET_JSON_spec_unblinded_signature (field,
+                                               &sig->signature);
+}
+
+struct GNUNET_JSON_Specification
+TALER_JSON_spec_blinded_token_issue_sig (
+  const char *field,
+  struct TALER_TokenIssueBlindSignatureP *sig)
+{
+  sig->signature = NULL;
+  return GNUNET_JSON_spec_blinded_signature (field,
+                                             &sig->signature);
+}
+
+
+struct GNUNET_JSON_Specification
+TALER_JSON_spec_token_envelope (const char *field,
+                                struct TALER_TokenEnvelopeP *env)
+{
+  env->blinded_pub = NULL;
+  return GNUNET_JSON_spec_blinded_message (field,
+                                           &env->blinded_pub);
+}
+
 
 /**
  * Parse given JSON object to denomination public key.
@@ -763,242 +792,13 @@ TALER_JSON_spec_denom_pub_cipher (const char *field,
 }
 
 
-/**
- * Parse given JSON object to denomination signature.
- *
- * @param cls closure, NULL
- * @param root the json object representing data
- * @param[out] spec where to write the data
- * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
- */
-static enum GNUNET_GenericReturnValue
-parse_denom_sig (void *cls,
-                 json_t *root,
-                 struct GNUNET_JSON_Specification *spec)
-{
-  struct TALER_DenominationSignature *denom_sig = spec->ptr;
-  struct GNUNET_CRYPTO_UnblindedSignature *unblinded_sig;
-  const char *cipher;
-  struct GNUNET_JSON_Specification dspec[] = {
-    GNUNET_JSON_spec_string ("cipher",
-                             &cipher),
-    GNUNET_JSON_spec_end ()
-  };
-  const char *emsg;
-  unsigned int eline;
-
-  (void) cls;
-  if (GNUNET_OK !=
-      GNUNET_JSON_parse (root,
-                         dspec,
-                         &emsg,
-                         &eline))
-  {
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
-  unblinded_sig = GNUNET_new (struct GNUNET_CRYPTO_UnblindedSignature);
-  unblinded_sig->cipher = string_to_cipher (cipher);
-  unblinded_sig->rc = 1;
-  switch (unblinded_sig->cipher)
-  {
-  case GNUNET_CRYPTO_BSA_INVALID:
-    break;
-  case GNUNET_CRYPTO_BSA_RSA:
-    {
-      struct GNUNET_JSON_Specification ispec[] = {
-        GNUNET_JSON_spec_rsa_signature (
-          "rsa_signature",
-          &unblinded_sig->details.rsa_signature),
-        GNUNET_JSON_spec_end ()
-      };
-
-      if (GNUNET_OK !=
-          GNUNET_JSON_parse (root,
-                             ispec,
-                             &emsg,
-                             &eline))
-      {
-        GNUNET_break_op (0);
-        GNUNET_free (unblinded_sig);
-        return GNUNET_SYSERR;
-      }
-      denom_sig->unblinded_sig = unblinded_sig;
-      return GNUNET_OK;
-    }
-  case GNUNET_CRYPTO_BSA_CS:
-    {
-      struct GNUNET_JSON_Specification ispec[] = {
-        GNUNET_JSON_spec_fixed_auto ("cs_signature_r",
-                                     &unblinded_sig->details.cs_signature.
-                                     r_point),
-        GNUNET_JSON_spec_fixed_auto ("cs_signature_s",
-                                     &unblinded_sig->details.cs_signature.
-                                     s_scalar),
-        GNUNET_JSON_spec_end ()
-      };
-
-      if (GNUNET_OK !=
-          GNUNET_JSON_parse (root,
-                             ispec,
-                             &emsg,
-                             &eline))
-      {
-        GNUNET_break_op (0);
-        GNUNET_free (unblinded_sig);
-        return GNUNET_SYSERR;
-      }
-      denom_sig->unblinded_sig = unblinded_sig;
-      return GNUNET_OK;
-    }
-  }
-  GNUNET_break_op (0);
-  GNUNET_free (unblinded_sig);
-  return GNUNET_SYSERR;
-}
-
-
-/**
- * Cleanup data left from parsing denomination public key.
- *
- * @param cls closure, NULL
- * @param[out] spec where to free the data
- */
-static void
-clean_denom_sig (void *cls,
-                 struct GNUNET_JSON_Specification *spec)
-{
-  struct TALER_DenominationSignature *denom_sig = spec->ptr;
-
-  (void) cls;
-  TALER_denom_sig_free (denom_sig);
-}
-
-
 struct GNUNET_JSON_Specification
 TALER_JSON_spec_denom_sig (const char *field,
                            struct TALER_DenominationSignature *sig)
 {
-  struct GNUNET_JSON_Specification ret = {
-    .parser = &parse_denom_sig,
-    .cleaner = &clean_denom_sig,
-    .field = field,
-    .ptr = sig
-  };
-
   sig->unblinded_sig = NULL;
-  return ret;
-}
-
-
-/**
- * Parse given JSON object to blinded denomination signature.
- *
- * @param cls closure, NULL
- * @param root the json object representing data
- * @param[out] spec where to write the data
- * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
- */
-static enum GNUNET_GenericReturnValue
-parse_blinded_denom_sig (void *cls,
-                         json_t *root,
-                         struct GNUNET_JSON_Specification *spec)
-{
-  struct TALER_BlindedDenominationSignature *denom_sig = spec->ptr;
-  struct GNUNET_CRYPTO_BlindedSignature *blinded_sig;
-  const char *cipher;
-  struct GNUNET_JSON_Specification dspec[] = {
-    GNUNET_JSON_spec_string ("cipher",
-                             &cipher),
-    GNUNET_JSON_spec_end ()
-  };
-  const char *emsg;
-  unsigned int eline;
-
-  (void) cls;
-  if (GNUNET_OK !=
-      GNUNET_JSON_parse (root,
-                         dspec,
-                         &emsg,
-                         &eline))
-  {
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
-  blinded_sig = GNUNET_new (struct GNUNET_CRYPTO_BlindedSignature);
-  blinded_sig->cipher = string_to_cipher (cipher);
-  blinded_sig->rc = 1;
-  switch (blinded_sig->cipher)
-  {
-  case GNUNET_CRYPTO_BSA_INVALID:
-    break;
-  case GNUNET_CRYPTO_BSA_RSA:
-    {
-      struct GNUNET_JSON_Specification ispec[] = {
-        GNUNET_JSON_spec_rsa_signature (
-          "blinded_rsa_signature",
-          &blinded_sig->details.blinded_rsa_signature),
-        GNUNET_JSON_spec_end ()
-      };
-
-      if (GNUNET_OK !=
-          GNUNET_JSON_parse (root,
-                             ispec,
-                             &emsg,
-                             &eline))
-      {
-        GNUNET_break_op (0);
-        GNUNET_free (blinded_sig);
-        return GNUNET_SYSERR;
-      }
-      denom_sig->blinded_sig = blinded_sig;
-      return GNUNET_OK;
-    }
-  case GNUNET_CRYPTO_BSA_CS:
-    {
-      struct GNUNET_JSON_Specification ispec[] = {
-        GNUNET_JSON_spec_uint32 ("b",
-                                 &blinded_sig->details.blinded_cs_answer.b),
-        GNUNET_JSON_spec_fixed_auto ("s",
-                                     &blinded_sig->details.blinded_cs_answer.
-                                     s_scalar),
-        GNUNET_JSON_spec_end ()
-      };
-
-      if (GNUNET_OK !=
-          GNUNET_JSON_parse (root,
-                             ispec,
-                             &emsg,
-                             &eline))
-      {
-        GNUNET_break_op (0);
-        GNUNET_free (blinded_sig);
-        return GNUNET_SYSERR;
-      }
-      denom_sig->blinded_sig = blinded_sig;
-      return GNUNET_OK;
-    }
-  }
-  GNUNET_break_op (0);
-  GNUNET_free (blinded_sig);
-  return GNUNET_SYSERR;
-}
-
-
-/**
- * Cleanup data left from parsing denomination public key.
- *
- * @param cls closure, NULL
- * @param[out] spec where to free the data
- */
-static void
-clean_blinded_denom_sig (void *cls,
-                         struct GNUNET_JSON_Specification *spec)
-{
-  struct TALER_BlindedDenominationSignature *denom_sig = spec->ptr;
-
-  (void) cls;
-  TALER_blinded_denom_sig_free (denom_sig);
+  return GNUNET_JSON_spec_unblinded_signature (field,
+                                               &sig->unblinded_sig);
 }
 
 
@@ -1007,131 +807,9 @@ TALER_JSON_spec_blinded_denom_sig (
   const char *field,
   struct TALER_BlindedDenominationSignature *sig)
 {
-  struct GNUNET_JSON_Specification ret = {
-    .parser = &parse_blinded_denom_sig,
-    .cleaner = &clean_blinded_denom_sig,
-    .field = field,
-    .ptr = sig
-  };
-
   sig->blinded_sig = NULL;
-  return ret;
-}
-
-
-/**
- * Parse given JSON object to blinded planchet.
- *
- * @param cls closure, NULL
- * @param root the json object representing data
- * @param[out] spec where to write the data
- * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
- */
-static enum GNUNET_GenericReturnValue
-parse_blinded_planchet (void *cls,
-                        json_t *root,
-                        struct GNUNET_JSON_Specification *spec)
-{
-  struct TALER_BlindedPlanchet *blinded_planchet = spec->ptr;
-  struct GNUNET_CRYPTO_BlindedMessage *blinded_message;
-  const char *cipher;
-  struct GNUNET_JSON_Specification dspec[] = {
-    GNUNET_JSON_spec_string ("cipher",
-                             &cipher),
-    GNUNET_JSON_spec_end ()
-  };
-  const char *emsg;
-  unsigned int eline;
-
-  (void) cls;
-  if (GNUNET_OK !=
-      GNUNET_JSON_parse (root,
-                         dspec,
-                         &emsg,
-                         &eline))
-  {
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
-  blinded_message = GNUNET_new (struct GNUNET_CRYPTO_BlindedMessage);
-  blinded_message->rc = 1;
-  blinded_message->cipher = string_to_cipher (cipher);
-  switch (blinded_message->cipher)
-  {
-  case GNUNET_CRYPTO_BSA_INVALID:
-    break;
-  case GNUNET_CRYPTO_BSA_RSA:
-    {
-      struct GNUNET_JSON_Specification ispec[] = {
-        GNUNET_JSON_spec_varsize (
-          "rsa_blinded_planchet",
-          &blinded_message->details.rsa_blinded_message.blinded_msg,
-          &blinded_message->details.rsa_blinded_message.blinded_msg_size),
-        GNUNET_JSON_spec_end ()
-      };
-
-      if (GNUNET_OK !=
-          GNUNET_JSON_parse (root,
-                             ispec,
-                             &emsg,
-                             &eline))
-      {
-        GNUNET_break_op (0);
-        GNUNET_free (blinded_message);
-        return GNUNET_SYSERR;
-      }
-      blinded_planchet->blinded_message = blinded_message;
-      return GNUNET_OK;
-    }
-  case GNUNET_CRYPTO_BSA_CS:
-    {
-      struct GNUNET_JSON_Specification ispec[] = {
-        GNUNET_JSON_spec_fixed_auto (
-          "cs_nonce",
-          &blinded_message->details.cs_blinded_message.nonce),
-        GNUNET_JSON_spec_fixed_auto (
-          "cs_blinded_c0",
-          &blinded_message->details.cs_blinded_message.c[0]),
-        GNUNET_JSON_spec_fixed_auto (
-          "cs_blinded_c1",
-          &blinded_message->details.cs_blinded_message.c[1]),
-        GNUNET_JSON_spec_end ()
-      };
-
-      if (GNUNET_OK !=
-          GNUNET_JSON_parse (root,
-                             ispec,
-                             &emsg,
-                             &eline))
-      {
-        GNUNET_break_op (0);
-        GNUNET_free (blinded_message);
-        return GNUNET_SYSERR;
-      }
-      blinded_planchet->blinded_message = blinded_message;
-      return GNUNET_OK;
-    }
-  }
-  GNUNET_break_op (0);
-  GNUNET_free (blinded_message);
-  return GNUNET_SYSERR;
-}
-
-
-/**
- * Cleanup data left from parsing blinded planchet.
- *
- * @param cls closure, NULL
- * @param[out] spec where to free the data
- */
-static void
-clean_blinded_planchet (void *cls,
-                        struct GNUNET_JSON_Specification *spec)
-{
-  struct TALER_BlindedPlanchet *blinded_planchet = spec->ptr;
-
-  (void) cls;
-  TALER_blinded_planchet_free (blinded_planchet);
+  return GNUNET_JSON_spec_blinded_signature (field,
+                                             &sig->blinded_sig);
 }
 
 
@@ -1139,15 +817,9 @@ struct GNUNET_JSON_Specification
 TALER_JSON_spec_blinded_planchet (const char *field,
                                   struct TALER_BlindedPlanchet *blinded_planchet)
 {
-  struct GNUNET_JSON_Specification ret = {
-    .parser = &parse_blinded_planchet,
-    .cleaner = &clean_blinded_planchet,
-    .field = field,
-    .ptr = blinded_planchet
-  };
-
   blinded_planchet->blinded_message = NULL;
-  return ret;
+  return GNUNET_JSON_spec_blinded_message (field,
+                                           &blinded_planchet->blinded_message);
 }
 
 
