@@ -145,37 +145,44 @@ TALER_JSON_pack_denom_sig (
   const char *name,
   const struct TALER_DenominationSignature *sig)
 {
-  const struct GNUNET_CRYPTO_UnblindedSignature *bs;
+  struct GNUNET_JSON_PackSpec ps = {
+    .field_name = name,
+  };
+  if (NULL == sig)
+    return ps;
+  return GNUNET_JSON_pack_unblinded_signature (name,
+                                               sig->unblinded_sig);
+}
+
+
+struct GNUNET_JSON_PackSpec
+TALER_JSON_pack_token_issue_sig (
+  const char *name,
+  const struct TALER_TokenIssueSignatureP *sig)
+{
+  struct GNUNET_JSON_PackSpec ps = {
+    .field_name = name,
+  };
+  if (NULL == sig)
+    return ps;
+  return GNUNET_JSON_pack_unblinded_signature (name,
+                                               sig->signature);
+}
+
+
+struct GNUNET_JSON_PackSpec
+TALER_JSON_pack_token_envelope (
+  const char *name,
+  const struct TALER_TokenEnvelopeP *envelope)
+{
   struct GNUNET_JSON_PackSpec ps = {
     .field_name = name,
   };
 
-  if (NULL == sig)
+  if (NULL == envelope)
     return ps;
-  bs = sig->unblinded_sig;
-  switch (bs->cipher)
-  {
-  case GNUNET_CRYPTO_BSA_INVALID:
-    break;
-  case GNUNET_CRYPTO_BSA_RSA:
-    ps.object = GNUNET_JSON_PACK (
-      GNUNET_JSON_pack_string ("cipher",
-                               "RSA"),
-      GNUNET_JSON_pack_rsa_signature ("rsa_signature",
-                                      bs->details.rsa_signature));
-    return ps;
-  case GNUNET_CRYPTO_BSA_CS:
-    ps.object = GNUNET_JSON_PACK (
-      GNUNET_JSON_pack_string ("cipher",
-                               "CS"),
-      GNUNET_JSON_pack_data_auto ("cs_signature_r",
-                                  &bs->details.cs_signature.r_point),
-      GNUNET_JSON_pack_data_auto ("cs_signature_s",
-                                  &bs->details.cs_signature.s_scalar));
-    return ps;
-  }
-  GNUNET_assert (0);
-  return ps;
+  return GNUNET_JSON_pack_blinded_message (name,
+                                           envelope->blinded_pub);
 }
 
 
@@ -265,44 +272,14 @@ TALER_JSON_pack_blinded_planchet (
   const char *name,
   const struct TALER_BlindedPlanchet *blinded_planchet)
 {
-  const struct GNUNET_CRYPTO_BlindedMessage *bm;
   struct GNUNET_JSON_PackSpec ps = {
     .field_name = name,
   };
 
   if (NULL == blinded_planchet)
     return ps;
-  bm = blinded_planchet->blinded_message;
-  switch (bm->cipher)
-  {
-  case GNUNET_CRYPTO_BSA_INVALID:
-    break;
-  case GNUNET_CRYPTO_BSA_RSA:
-    ps.object = GNUNET_JSON_PACK (
-      GNUNET_JSON_pack_string ("cipher",
-                               "RSA"),
-      GNUNET_JSON_pack_data_varsize (
-        "rsa_blinded_planchet",
-        bm->details.rsa_blinded_message.blinded_msg,
-        bm->details.rsa_blinded_message.blinded_msg_size));
-    return ps;
-  case GNUNET_CRYPTO_BSA_CS:
-    ps.object = GNUNET_JSON_PACK (
-      GNUNET_JSON_pack_string ("cipher",
-                               "CS"),
-      GNUNET_JSON_pack_data_auto (
-        "cs_nonce",
-        &bm->details.cs_blinded_message.nonce),
-      GNUNET_JSON_pack_data_auto (
-        "cs_blinded_c0",
-        &bm->details.cs_blinded_message.c[0]),
-      GNUNET_JSON_pack_data_auto (
-        "cs_blinded_c1",
-        &bm->details.cs_blinded_message.c[1]));
-    return ps;
-  }
-  GNUNET_assert (0);
-  return ps;
+  return GNUNET_JSON_pack_blinded_message (name,
+                                           blinded_planchet->blinded_message);
 }
 
 
