@@ -67,25 +67,40 @@ struct CoinCandidate
  */
 struct CSRClosure
 {
-  /* Points to the actual candidate in CoinData.coin_candidates, to continue
-   * to build its contents based on the results from /csr-withdraw */
+  /**
+   * Points to the actual candidate in CoinData.coin_candidates, to continue
+   * to build its contents based on the results from /csr-withdraw
+   */
   struct CoinCandidate *candidate;
 
-  /* The planchet to finally generate.  Points to the corresponding candidate
-   * in CoindData.planchet_details */
+  /**
+   * The planchet to finally generate.  Points to the corresponding candidate
+   * in CoindData.planchet_details
+   */
   struct TALER_PlanchetDetail *planchet;
 
-  /* Handler to the originating call to /age-withdraw, needed to either
+  /**
+   * Handler to the originating call to /age-withdraw, needed to either
    * cancel the running age-withdraw request (on failure of the current call
    * to /csr-withdraw), or to eventually perform the protocol, once all
-   * csr-withdraw requests have successfully finished. */
+   * csr-withdraw requests have successfully finished.
+   */
   struct TALER_EXCHANGE_AgeWithdrawHandle *age_withdraw_handle;
 
-  /* Denomination information, needed for CS coins for the
-   * step after /csr-withdraw */
+  /**
+   * Session nonce.
+   */
+  union GNUNET_CRYPTO_BlindSessionNonce nonce;
+
+  /**
+   * Denomination information, needed for CS coins for the
+   * step after /csr-withdraw
+   */
   const struct TALER_EXCHANGE_DenomPublicKey *denom_pub;
 
-  /* Handler for the CS R request */
+  /**
+   * Handler for the CS R request
+   */
   struct TALER_EXCHANGE_CsRWithdrawHandle *csr_withdraw_handle;
 };
 
@@ -146,7 +161,7 @@ struct TALER_EXCHANGE_AgeWithdrawBlindedHandle
   struct TALER_EXCHANGE_Keys *keys;
 
   /**
-   * The age mask, extacted from the denominations.
+   * The age mask, extracted from the denominations.
    * MUST be the same for all denominations
    *
    */
@@ -248,7 +263,7 @@ struct TALER_EXCHANGE_AgeWithdrawHandle
   struct TALER_EXCHANGE_Keys *keys;
 
   /**
-   * The age mask, extacted from the denominations.
+   * The age mask, extracted from the denominations.
    * MUST be the same for all denominations
    *
    */
@@ -396,27 +411,6 @@ handle_reserve_age_withdraw_blinded_finished (
     GNUNET_assert (NULL == awbh->callback);
     TALER_EXCHANGE_age_withdraw_blinded_cancel (awbh);
     return;
-  case MHD_HTTP_UNAVAILABLE_FOR_LEGAL_REASONS:
-    /* only validate reply is well-formed */
-    {
-      uint64_t ptu;
-      struct GNUNET_JSON_Specification spec[] = {
-        GNUNET_JSON_spec_uint64 ("legitimization_uuid",
-                                 &ptu),
-        GNUNET_JSON_spec_end ()
-      };
-
-      if (GNUNET_OK !=
-          GNUNET_JSON_parse (j_response,
-                             spec,
-                             NULL, NULL))
-      {
-        GNUNET_break_op (0);
-        awbr.hr.http_status = 0;
-        awbr.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
-        break;
-      }
-    }
   case MHD_HTTP_BAD_REQUEST:
     /* This should never happen, either us or the exchange is buggy
        (or API version conflict); just pass JSON reply to the application */
@@ -452,6 +446,27 @@ handle_reserve_age_withdraw_blinded_finished (
     awbr.hr.ec = TALER_JSON_get_error_code (j_response);
     awbr.hr.hint = TALER_JSON_get_error_hint (j_response);
     break;
+  case MHD_HTTP_UNAVAILABLE_FOR_LEGAL_REASONS:
+    /* only validate reply is well-formed */
+    {
+      uint64_t ptu;
+      struct GNUNET_JSON_Specification spec[] = {
+        GNUNET_JSON_spec_uint64 ("requirement_row",
+                                 &ptu),
+        GNUNET_JSON_spec_end ()
+      };
+
+      if (GNUNET_OK !=
+          GNUNET_JSON_parse (j_response,
+                             spec,
+                             NULL, NULL))
+      {
+        GNUNET_break_op (0);
+        awbr.hr.http_status = 0;
+        awbr.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
+        break;
+      }
+    }
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
@@ -491,9 +506,9 @@ perform_protocol (
       GNUNET_break (! (cond)); \
       goto ERROR; \
     } \
-  } while(0)
+  } while (0)
 
-  struct GNUNET_HashContext *coins_hctx;
+  struct GNUNET_HashContext *coins_hctx = NULL;
   json_t *j_denoms = NULL;
   json_t *j_array_candidates = NULL;
   json_t *j_request_body = NULL;
@@ -552,7 +567,7 @@ perform_protocol (
         GNUNET_JSON_pack_data_auto (NULL,
                                     denom_h));
       FAIL_IF (NULL == jdenom);
-      FAIL_IF (0 < json_array_append_new (j_denoms,
+      FAIL_IF (0 > json_array_append_new (j_denoms,
                                           jdenom));
 
       /* Build the candidate array */
@@ -571,7 +586,7 @@ perform_protocol (
               &planchet->blinded_planchet));
 
           FAIL_IF (NULL == jc);
-          FAIL_IF (0 < json_array_append_new (j_can,
+          FAIL_IF (0 > json_array_append_new (j_can,
                                               jc));
 
           TALER_coin_ev_hash (&planchet->blinded_planchet,
@@ -583,7 +598,7 @@ perform_protocol (
                                            sizeof(bch));
         }
 
-        FAIL_IF (0 < json_array_append_new (j_array_candidates,
+        FAIL_IF (0 > json_array_append_new (j_array_candidates,
                                             j_can));
       }
     }
@@ -592,6 +607,7 @@ perform_protocol (
   /* Build the hash of the commitment */
   GNUNET_CRYPTO_hash_context_finish (coins_hctx,
                                      &awbh->h_commitment.hash);
+  coins_hctx = NULL;
 
   /* Sign the request */
   TALER_wallet_age_withdraw_sign (&awbh->h_commitment,
@@ -638,6 +654,8 @@ ERROR:
     json_decref (j_request_body);
   if (NULL != curlh)
     curl_easy_cleanup (curlh);
+  if (NULL != coins_hctx)
+    GNUNET_CRYPTO_hash_context_abort (coins_hctx);
   TALER_EXCHANGE_age_withdraw_blinded_cancel (awbh);
   return;
 #undef FAIL_IF
@@ -663,12 +681,14 @@ copy_results (
   struct TALER_EXCHANGE_AgeWithdrawResponse resp = {
     .hr = awbr->hr,
     .details = {
-      .ok = { .noreveal_index = awbr->details.ok.noreveal_index,
-              .h_commitment = awbr->details.ok.h_commitment,
-              .exchange_pub = awbr->details.ok.exchange_pub,
-              .num_coins = awh->num_coins,
-              .coin_details = details,
-              .blinded_coin_hs = blinded_coin_hs},
+      .ok = {
+        .noreveal_index = awbr->details.ok.noreveal_index,
+        .h_commitment = awbr->details.ok.h_commitment,
+        .exchange_pub = awbr->details.ok.exchange_pub,
+        .num_coins = awh->num_coins,
+        .coin_details = details,
+        .blinded_coin_hs = blinded_coin_hs
+      },
     },
   };
 
@@ -678,10 +698,8 @@ copy_results (
     details[n].planchet = awh->coin_data[n].planchet_details[k];
     blinded_coin_hs[n] = awh->coin_data[n].coin_candidates[k].blinded_coin_h;
   }
-
   awh->callback (awh->callback_cls,
                  &resp);
-
   awh->callback = NULL;
 }
 
@@ -794,9 +812,11 @@ csr_withdraw_done (
     {
       bool success = false;
       /* Complete the initialization of the coin with CS denomination */
-      can->details.alg_values = csrr->details.ok.alg_values;
-      GNUNET_assert (can->details.alg_values.cipher
-                     == TALER_DENOMINATION_CS);
+
+      TALER_denom_ewv_copy (&can->details.alg_values,
+                            &csrr->details.ok.alg_values);
+      GNUNET_assert (can->details.alg_values.blinding_inputs->cipher
+                     == GNUNET_CRYPTO_BSA_CS);
       TALER_planchet_setup_coin_priv (&can->secret,
                                       &can->details.alg_values,
                                       &can->details.coin_priv);
@@ -810,27 +830,21 @@ csr_withdraw_done (
             TALER_planchet_prepare (&csr->denom_pub->key,
                                     &can->details.alg_values,
                                     &can->details.blinding_key,
+                                    &csr->nonce,
                                     &can->details.coin_priv,
                                     &can->details.h_age_commitment,
                                     &can->details.h_coin_pub,
                                     planchet))
         {
           GNUNET_break (0);
-          TALER_EXCHANGE_age_withdraw_cancel (awh);
           break;
         }
 
-        if (GNUNET_OK !=
-            TALER_coin_ev_hash (&planchet->blinded_planchet,
-                                &planchet->denom_pub_hash,
-                                &can->blinded_coin_h))
-        {
-          GNUNET_break (0);
-          TALER_EXCHANGE_age_withdraw_cancel (awh);
-          break;
-        }
+        TALER_coin_ev_hash (&planchet->blinded_planchet,
+                            &planchet->denom_pub_hash,
+                            &can->blinded_coin_h);
         success = true;
-      } while(0);
+      } while (0);
 
       awh->csr.pending--;
 
@@ -843,7 +857,6 @@ csr_withdraw_done (
   default:
     break;
   }
-
   TALER_EXCHANGE_age_withdraw_cancel (awh);
 }
 
@@ -875,7 +888,7 @@ prepare_coins (
       GNUNET_break (! (cond)); \
       goto ERROR; \
     } \
-  } while(0)
+  } while (0)
 
   GNUNET_assert (0 < num_coins);
   awh->age_mask = coin_inputs[0].denom_pub->key.age_mask;
@@ -887,13 +900,12 @@ prepare_coins (
   {
     struct CoinData *cd = &awh->coin_data[i];
     const struct TALER_EXCHANGE_AgeWithdrawCoinInput *input = &coin_inputs[i];
-    cd->denom_pub = *input->denom_pub;
 
+    cd->denom_pub = *input->denom_pub;
     /* The mask must be the same for all coins */
     FAIL_IF (awh->age_mask.bits != input->denom_pub->key.age_mask.bits);
-
-    TALER_denom_pub_deep_copy (&cd->denom_pub.key,
-                               &input->denom_pub->key);
+    TALER_denom_pub_copy (&cd->denom_pub.key,
+                          &input->denom_pub->key);
 
     for (uint8_t k = 0; k < TALER_CNC_KAPPA; k++)
     {
@@ -912,35 +924,32 @@ prepare_coins (
       TALER_age_commitment_hash (&can->details.age_commitment_proof.commitment,
                                  &can->details.h_age_commitment);
 
-      switch (input->denom_pub->key.cipher)
+      switch (input->denom_pub->key.bsign_pub_key->cipher)
       {
-      case TALER_DENOMINATION_RSA:
+      case GNUNET_CRYPTO_BSA_RSA:
+        TALER_denom_ewv_copy (&can->details.alg_values,
+                              TALER_denom_ewv_rsa_singleton ());
+        TALER_planchet_setup_coin_priv (&can->secret,
+                                        &can->details.alg_values,
+                                        &can->details.coin_priv);
+        TALER_planchet_blinding_secret_create (&can->secret,
+                                               &can->details.alg_values,
+                                               &can->details.blinding_key);
+        FAIL_IF (GNUNET_OK !=
+                 TALER_planchet_prepare (&cd->denom_pub.key,
+                                         &can->details.alg_values,
+                                         &can->details.blinding_key,
+                                         NULL,
+                                         &can->details.coin_priv,
+                                         &can->details.h_age_commitment,
+                                         &can->details.h_coin_pub,
+                                         planchet));
+        TALER_coin_ev_hash (&planchet->blinded_planchet,
+                            &planchet->denom_pub_hash,
+                            &can->blinded_coin_h);
+        break;
+      case GNUNET_CRYPTO_BSA_CS:
         {
-          can->details.alg_values.cipher = TALER_DENOMINATION_RSA;
-          TALER_planchet_setup_coin_priv (&can->secret,
-                                          &can->details.alg_values,
-                                          &can->details.coin_priv);
-          TALER_planchet_blinding_secret_create (&can->secret,
-                                                 &can->details.alg_values,
-                                                 &can->details.blinding_key);
-          FAIL_IF (GNUNET_OK !=
-                   TALER_planchet_prepare (&cd->denom_pub.key,
-                                           &can->details.alg_values,
-                                           &can->details.blinding_key,
-                                           &can->details.coin_priv,
-                                           &can->details.h_age_commitment,
-                                           &can->details.h_coin_pub,
-                                           planchet));
-          FAIL_IF (GNUNET_OK !=
-                   TALER_coin_ev_hash (&planchet->blinded_planchet,
-                                       &planchet->denom_pub_hash,
-                                       &can->blinded_coin_h));
-          break;
-        }
-      case TALER_DENOMINATION_CS:
-        {
-          can->details.alg_values.cipher = TALER_DENOMINATION_CS;
-
           struct CSRClosure *cls = &cd->csr_cls[k];
           /**
            * Save the handler and the denomination for the callback
@@ -949,21 +958,15 @@ prepare_coins (
           cls->candidate = can;
           cls->planchet = planchet;
           cls->denom_pub = &cd->denom_pub;
-
           TALER_cs_withdraw_nonce_derive (
             &can->secret,
-            &planchet->blinded_planchet.details.cs_blinded_planchet.nonce);
-
-          /* Note that we only initialize the first half
-             of the blinded_planchet here; the other part
-             will be done after the /csr-withdraw request! */
-          planchet->blinded_planchet.cipher = TALER_DENOMINATION_CS;
+            &cls->nonce.cs_nonce);
           cls->csr_withdraw_handle =
             TALER_EXCHANGE_csr_withdraw (
               awh->curl_ctx,
               awh->exchange_url,
               &cd->denom_pub,
-              &planchet->blinded_planchet.details.cs_blinded_planchet.nonce,
+              &cls->nonce.cs_nonce,
               &csr_withdraw_done,
               cls);
           FAIL_IF (NULL == cls->csr_withdraw_handle);
@@ -1013,7 +1016,10 @@ TALER_EXCHANGE_age_withdraw (
   if (GNUNET_OK != prepare_coins (awh,
                                   num_coins,
                                   coin_inputs))
+  {
+    GNUNET_free (awh);
     return NULL;
+  }
 
   /* If there were no CS denominations, we can now perform the actual
    * age-withdraw protocol.  Otherwise, there are calls to /csr-withdraw
@@ -1040,6 +1046,7 @@ TALER_EXCHANGE_age_withdraw_cancel (
     {
       struct TALER_PlanchetDetail *planchet = &cd->planchet_details[k];
       struct CSRClosure *cls = &cd->csr_cls[k];
+      struct CoinCandidate *can = &cd->coin_candidates[k];
 
       if (NULL != cls->csr_withdraw_handle)
       {
@@ -1047,6 +1054,7 @@ TALER_EXCHANGE_age_withdraw_cancel (
         cls->csr_withdraw_handle = NULL;
       }
       TALER_blinded_planchet_free (&planchet->blinded_planchet);
+      TALER_denom_ewv_free (&can->details.alg_values);
     }
     TALER_denom_pub_free (&cd->denom_pub.key);
   }
