@@ -177,7 +177,6 @@ handle_deposit_wtid_finished (void *cls,
     {
       /* Transaction known, but not executed yet */
       bool no_legi = false;
-      uint32_t state32;
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_timestamp ("execution_time",
                                     &dr.details.accepted.execution_time),
@@ -185,8 +184,8 @@ handle_deposit_wtid_finished (void *cls,
           GNUNET_JSON_spec_uint64 ("requirement_row",
                                    &dr.details.accepted.requirement_row),
           &no_legi),
-        GNUNET_JSON_spec_uint32 ("aml_decision",
-                                 &state32),
+        TALER_JSON_spec_aml_decision ("aml_decision",
+                                      &dr.details.accepted.aml_decision),
         GNUNET_JSON_spec_bool ("kyc_ok",
                                &dr.details.accepted.kyc_ok),
         GNUNET_JSON_spec_end ()
@@ -202,8 +201,6 @@ handle_deposit_wtid_finished (void *cls,
         dr.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
         break;
       }
-      dr.details.accepted.aml_decision
-        = (enum TALER_AmlDecisionState) state32;
       if (no_legi)
         dr.details.accepted.requirement_row = 0;
       dwh->cb (dwh->cb_cls,
@@ -275,6 +272,9 @@ TALER_EXCHANGE_deposits_get (
                 + sizeof (struct TALER_MerchantPublicKeyP)
                 + sizeof (struct TALER_PrivateContractHashP)
                 + sizeof (struct TALER_MerchantSignatureP)) * 2 + 48];
+  unsigned int tms
+    = (unsigned int) timeout.rel_value_us
+      / GNUNET_TIME_UNIT_MILLISECONDS.rel_value_us;
 
   GNUNET_CRYPTO_eddsa_key_get_public (&merchant_priv->eddsa_priv,
                                       &merchant.eddsa_pub);
@@ -326,10 +326,8 @@ TALER_EXCHANGE_deposits_get (
       GNUNET_snprintf (
         timeout_str,
         sizeof (timeout_str),
-        "%llu",
-        (unsigned long long) (
-          timeout.rel_value_us
-          / GNUNET_TIME_UNIT_MILLISECONDS.rel_value_us));
+        "%u",
+        tms);
     }
 
     GNUNET_snprintf (arg_str,
@@ -340,7 +338,7 @@ TALER_EXCHANGE_deposits_get (
                      chash_str,
                      cpub_str,
                      msig_str,
-                     GNUNET_TIME_relative_is_zero (timeout)
+                     0 == tms
                      ? ""
                      : "&timeout_ms=",
                      timeout_str);
@@ -368,6 +366,13 @@ TALER_EXCHANGE_deposits_get (
     GNUNET_free (dwh);
     return NULL;
   }
+  if (0 != tms)
+  {
+    GNUNET_break (CURLE_OK ==
+                  curl_easy_setopt (eh,
+                                    CURLOPT_TIMEOUT_MS,
+                                    (long) (tms + 100L)));
+  }
   dwh->job = GNUNET_CURL_job_add (ctx,
                                   eh,
                                   &handle_deposit_wtid_finished,
