@@ -48,7 +48,7 @@ struct TALER_TESTING_Credentials cred;
 
 /**
  * Some tests behave differently when using CS as we cannot
- * re-use the coin private key for different denominations
+ * reuse the coin private key for different denominations
  * due to the derivation of it with the /csr values. Hence
  * some tests behave differently in CS mode, hence this
  * flag.
@@ -254,12 +254,12 @@ run (void *cls,
     TALER_TESTING_cmd_status (
       "pull-check-post-merge-reserve-balance-get",
       "create-reserve-1",
-      "EUR:2.01",
+      "EUR:2.02",
       MHD_HTTP_OK),
     TALER_TESTING_cmd_reserve_history (
       "push-check-post-merge-reserve-balance-post",
       "create-reserve-1",
-      "EUR:2.01",
+      "EUR:2.02",
       MHD_HTTP_OK),
     TALER_TESTING_cmd_purse_deposit_coins (
       "purse-deposit-coins-idempotent",
@@ -396,7 +396,7 @@ run (void *cls,
                                     "EUR:0",
                                     GNUNET_TIME_UNIT_YEARS,
                                     5, /* min purses */
-                                    MHD_HTTP_PAYMENT_REQUIRED, // FIXME: or CONFLICT?
+                                    MHD_HTTP_PAYMENT_REQUIRED,
                                     NULL,
                                     NULL),
     TALER_TESTING_cmd_reserve_open ("reserve-open-101-ok-a",
