@@ -184,7 +184,10 @@ run (void *cls,
                                         "kyc-provider-test-oauth2",
                                         "bad",
                                         MHD_HTTP_FORBIDDEN),
-    TALER_TESTING_cmd_proof_kyc_oauth2 ("proof-kyc-fail",
+    TALER_TESTING_cmd_check_kyc_get ("check-kyc-deposit-again",
+                                     "track-deposit-kyc-ready",
+                                     MHD_HTTP_ACCEPTED),
+    TALER_TESTING_cmd_proof_kyc_oauth2 ("proof-kyc-pass",
                                         "track-deposit-kyc-ready",
                                         "kyc-provider-test-oauth2",
                                         "pass",
