@@ -165,11 +165,13 @@ close_run (void *cls,
   }
   GNUNET_CRYPTO_eddsa_key_get_public (&ss->reserve_priv->eddsa_priv,
                                       &ss->reserve_pub.eddsa_pub);
-  ss->rsh = TALER_EXCHANGE_reserves_close (is->exchange,
-                                           ss->reserve_priv,
-                                           ss->target_account,
-                                           &reserve_close_cb,
-                                           ss);
+  ss->rsh = TALER_EXCHANGE_reserves_close (
+    TALER_TESTING_interpreter_get_context (is),
+    TALER_TESTING_get_exchange_url (is),
+    ss->reserve_priv,
+    ss->target_account,
+    &reserve_close_cb,
+    ss);
 }
 
 
@@ -188,10 +190,8 @@ close_cleanup (void *cls,
 
   if (NULL != ss->rsh)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Command %u (%s) did not complete\n",
-                ss->is->ip,
-                cmd->label);
+    TALER_TESTING_command_incomplete (ss->is,
+                                      cmd->label);
     TALER_EXCHANGE_reserves_close_cancel (ss->rsh);
     ss->rsh = NULL;
   }
