@@ -136,6 +136,13 @@ static struct GNUNET_TIME_Timestamp now_tmp;
  */
 static char *keydir;
 
+/**
+ * Name of the configuration section prefix to use.  Usually either "taler-exchange" or
+ * "donau". The actual configuration section will then be
+ * "$SECTION-secmod-eddsa".
+ */
+static char *section;
+
 /**
  * How much should coin creation duration overlap
  * with the next key?  Basically, the starting time of two
@@ -584,11 +591,11 @@ eddsa_client_init (struct TES_Client *client)
 static enum GNUNET_GenericReturnValue
 eddsa_update_client_keys (struct TES_Client *client)
 {
+  GNUNET_assert (0 == pthread_mutex_lock (&keys_lock));
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Updating client %p to generation %llu\n",
               client,
               (unsigned long long) key_gen);
-  GNUNET_assert (0 == pthread_mutex_lock (&keys_lock));
   for (struct Key *key = keys_head;
        NULL != key;
        key = key->next)
@@ -991,39 +998,48 @@ import_key (void *cls,
 static enum GNUNET_GenericReturnValue
 load_durations (const struct GNUNET_CONFIGURATION_Handle *cfg)
 {
+  char *secname;
+
+  GNUNET_asprintf (&secname,
+                   "%s-secmod-eddsa",
+                   section);
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_time (cfg,
-                                           "taler-exchange-secmod-eddsa",
+                                           secname,
                                            "OVERLAP_DURATION",
                                            &overlap_duration))
   {
+    GNUNET_free (secname);
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               "taler-exchange-secmod-eddsa",
+                               secname,
                                "OVERLAP_DURATION");
     return GNUNET_SYSERR;
   }
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_time (cfg,
-                                           "taler-exchange-secmod-eddsa",
+                                           secname,
                                            "DURATION",
                                            &duration))
   {
+    GNUNET_free (secname);
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               "taler-exchange-secmod-eddsa",
+                               secname,
                                "DURATION");
     return GNUNET_SYSERR;
   }
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_time (cfg,
-                                           "taler-exchange-secmod-eddsa",
+                                           secname,
                                            "LOOKAHEAD_SIGN",
                                            &lookahead_sign))
   {
+    GNUNET_free (secname);
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               "taler-exchange-secmod-eddsa",
+                               secname,
                                "LOOKAHEAD_SIGN");
     return GNUNET_SYSERR;
   }
+  GNUNET_free (secname);
   return GNUNET_OK;
 }
 
@@ -1065,6 +1081,7 @@ run (void *cls,
     .updater = eddsa_update_client_keys,
     .init = eddsa_client_init
   };
+  char *secname;
 
   (void) cls;
   (void) args;
@@ -1079,6 +1096,9 @@ run (void *cls,
     /* get current time again, we may be timetraveling! */
     now = GNUNET_TIME_timestamp_get ();
   }
+  GNUNET_asprintf (&secname,
+                   "%s-secmod-eddsa",
+                   section);
   if (GNUNET_OK !=
       load_durations (cfg))
   {
@@ -1087,21 +1107,31 @@ run (void *cls,
   }
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_filename (cfg,
-                                               "taler-exchange-secmod-eddsa",
+                                               secname,
                                                "KEY_DIR",
                                                &keydir))
   {
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               "taler-exchange-secmod-eddsa",
+                               secname,
                                "KEY_DIR");
+    GNUNET_free (secname);
     global_ret = EXIT_NOTCONFIGURED;
     return;
   }
+  GNUNET_free (secname);
   GNUNET_SCHEDULER_add_shutdown (&do_shutdown,
                                  NULL);
-  global_ret = TES_listen_start (cfg,
-                                 "taler-exchange-secmod-eddsa",
-                                 &cb);
+  {
+    char *secname;
+
+    GNUNET_asprintf (&secname,
+                     "%s-secmod-eddsa",
+                     section);
+    global_ret = TES_listen_start (cfg,
+                                   secname,
+                                   &cb);
+    GNUNET_free (secname);
+  }
   if (0 != global_ret)
     return;
   /* Load keys */
@@ -1144,6 +1174,11 @@ main (int argc,
       char **argv)
 {
   struct GNUNET_GETOPT_CommandLineOption options[] = {
+    GNUNET_GETOPT_option_string ('s',
+                                 "section",
+                                 "SECTION",
+                                 "name of the configuration section prefix to use, default is 'taler'",
+                                 &section),
     GNUNET_GETOPT_option_timetravel ('T',
                                      "timetravel"),
     GNUNET_GETOPT_option_timestamp ('t',
@@ -1157,7 +1192,7 @@ main (int argc,
 
   /* Restrict permissions for the key files that we create. */
   (void) umask (S_IWGRP | S_IROTH | S_IWOTH | S_IXOTH);
-
+  section = GNUNET_strdup ("taler-exchange");
   /* force linker to link against libtalerutil; if we do
    not do this, the linker may "optimize" libtalerutil
    away and skip #TALER_OS_init(), which we do need */
